<?php
/**
 * This file is part of the official Amazon Checkout extension for Magento 1.x
 *
 * (c) 2020 creativestyle GmbH. All Rights reserved
 *
 * Distribution of the derivatives reusing, transforming or being built upon
 * this software, is not allowed without explicit written permission granted
 * by creativestyle GmbH
 *
 * @category   Creativestyle
 * @package    Creativestyle_AmazonCheckout
 * @copyright  2020 creativestyle GmbH
 * @author     Marek Stefan <ticket@creativestyle.de>
 */

class Creativestyle_AmazonCheckout_Model_Checkout
{
    /**
     * @var string
     */
    protected $_amazonCheckoutSessionId;

    /**
     * @var null|Creativestyle_AmazonCheckout_Model_Api_CheckoutSessionInterface
     */
    protected $_amazonCheckoutSession = null;

    /**
     * @var Mage_Sales_Model_Quote
     */
    protected $_quote = null;

    /**
     * Return instance of Amazon Pay API adapter
     *
     * @return Creativestyle_AmazonCheckout_Model_Api
     */
    protected function _getApi()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Api $api */
        $api = Mage::getSingleton('amazonpay/api');
        return $api;
    }

    /**
     * Return instance of Amazon -> Magento data mapper
     *
     * @return Creativestyle_AmazonCheckout_Model_DataMapper
     */
    protected function _getDataMapper()
    {
        /** @var Creativestyle_AmazonCheckout_Model_DataMapper $dataMapper */
        $dataMapper = Mage::getSingleton('amazonpay/dataMapper');
        return $dataMapper;
    }

    /**
     * Return logger instance
     *
     * @return Creativestyle_AmazonCheckout_Model_Logger
     */
    protected function _getLogger()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Logger $logger */
        $logger = Mage::getSingleton('amazonpay/logger');
        return $logger;
    }

    /**
     * @return bool
     * @throws Creativestyle_AmazonCheckout_Exception_InvalidArgument
     */
    protected function _validateAmazonCheckoutSessionId()
    {
        if (empty($this->_amazonCheckoutSessionId)) {
            throw new Creativestyle_AmazonCheckout_Exception_InvalidArgument('Checkout session ID is not set');
        }

        return true;
    }

    /**
     * Prepare quote for guest checkout order submit
     *
     * @return $this
     */
    protected function _prepareGuestQuote()
    {
        $quote = $this->getQuote();
        $quote->setCustomerId(null)
            ->setCustomerEmail($quote->getBillingAddress()->getEmail())
            ->setCustomerIsGuest(true)
            ->setCustomerGroupId(Mage_Customer_Model_Group::NOT_LOGGED_IN_ID);
        return $this;
    }

    /**
     * Return instance of the Magento checkout session
     *
     * @return Mage_Checkout_Model_Session
     */
    public function getSession()
    {
        /** @var Mage_Checkout_Model_Session $checkoutSession */
        $checkoutSession = Mage::getSingleton('checkout/session');
        return $checkoutSession;
    }

    /**
     * Quote object getter
     *
     * @return Mage_Sales_Model_Quote
     */
    public function getQuote()
    {
        if ($this->_quote === null) {
            return $this->getSession()->getQuote();
        }
        return $this->_quote;
    }

    /**
     * Declare checkout quote instance
     *
     * @param Mage_Sales_Model_Quote $quote
     * @return $this
     */
    public function setQuote(Mage_Sales_Model_Quote $quote)
    {
        $this->_quote = $quote;
        return $this;
    }

    /**
     * Set ID of the Amazon checkout session object
     *
     * @param string $amazonCheckoutSessionId
     * @return $this
     */
    public function setAmazonCheckoutSessionId($amazonCheckoutSessionId)
    {
        $this->_amazonCheckoutSessionId = $amazonCheckoutSessionId;
        return $this;
    }

    /**
     * @return $this
     * @throws Creativestyle_AmazonCheckout_Exception_Runtime_Api
     */
    public function createAmazonCheckoutSession()
    {
        $this->_amazonCheckoutSession = $this->_getApi()->createCheckoutSession();
        $this->setAmazonCheckoutSessionId($this->_amazonCheckoutSession->getCheckoutSessionId());
        return $this;
    }

    /**
     * @return Creativestyle_AmazonCheckout_Model_Api_CheckoutSessionInterface
     * @throws Creativestyle_AmazonCheckout_Exception_InvalidArgument
     * @throws Creativestyle_AmazonCheckout_Exception_Runtime_Api
     */
    public function getAmazonCheckoutSession()
    {
        $this->_validateAmazonCheckoutSessionId();

        if (null === $this->_amazonCheckoutSession) {
            $this->_amazonCheckoutSession = $this->_getApi()->getCheckoutSession($this->_amazonCheckoutSessionId);
        }

        return $this->_amazonCheckoutSession;
    }

    /**
     * @return $this
     * @throws Creativestyle_AmazonCheckout_Exception_InvalidArgument
     * @throws Creativestyle_AmazonCheckout_Exception_Runtime_Api
     */
    public function updateAmazonCheckoutSession()
    {
        $this->_validateAmazonCheckoutSessionId();

        $this->getQuote()->reserveOrderId()->save();

        $this->_amazonCheckoutSession = $this->_getApi()->updateCheckoutSession(
            $this->_amazonCheckoutSessionId,
            $this->getQuote()->getBaseGrandTotal(),
            $this->getQuote()->getBaseCurrencyCode(),
            $this->getQuote()->getReservedOrderId()
        );

        return $this;
    }

    /**
     * @return $this
     * @throws Creativestyle_AmazonCheckout_Exception_InvalidArgument
     * @throws Creativestyle_AmazonCheckout_Exception_Runtime_Api
     */
    public function importCustomerData()
    {
        $checkoutSession = $this->getAmazonCheckoutSession();
        $this->saveCustomerData(
            $checkoutSession->getBuyerEmail(),
            $checkoutSession->getBillingAddress(),
            $checkoutSession->getShippingAddress()
        );

        return $this;
    }

    /**
     * @param string $email
     * @param Creativestyle_AmazonCheckout_Model_Api_AddressInterface|null $amazonBillingAddress
     * @param Creativestyle_AmazonCheckout_Model_Api_AddressInterface|null $amazonShippingAddress
     */
    public function saveCustomerData(
        $email,
        Creativestyle_AmazonCheckout_Model_Api_AddressInterface $amazonBillingAddress = null,
        Creativestyle_AmazonCheckout_Model_Api_AddressInterface $amazonShippingAddress = null
    ) {
        $quote = $this->getQuote();

        $quote->setCustomerEmail($email);
        $billingAddress = $quote->getBillingAddress();
        $this->_getDataMapper()->mapAmazonPayAddress($amazonBillingAddress, $billingAddress);
        $billingAddress
            ->setEmail($email)
            ->setCustomerAddressId(null)
            ->setSaveInAddressBook(0);

        $paymentMethod = 'amazon_checkout';

        if (!$quote->isVirtual()) {
            $shippingAddress = $quote->getShippingAddress();
            $this->_getDataMapper()->mapAmazonPayAddress($amazonShippingAddress, $shippingAddress);
            $shippingAddress
                ->setCustomerAddressId(null)
                ->setSaveInAddressBook(0)
                ->setPaymentMethod($paymentMethod)
                ->setCollectShippingRates(true);
        } else {
            $billingAddress->setPaymentMethod($paymentMethod);
        }

        $quote->getPayment()->importData(array(
            'method' => $paymentMethod,
            'checks' => Mage_Payment_Model_Method_Abstract::CHECK_USE_FOR_COUNTRY
                | Mage_Payment_Model_Method_Abstract::CHECK_USE_FOR_CURRENCY
                | Mage_Payment_Model_Method_Abstract::CHECK_ORDER_TOTAL_MIN_MAX
                | Mage_Payment_Model_Method_Abstract::CHECK_ZERO_TOTAL
        ));

        $quote->collectTotals();
        $quote->save();
    }

    /**
     * @param string $shippingMethod
     * @throws Creativestyle_AmazonCheckout_Exception
     */
    public function saveShippingMethod($shippingMethod)
    {
        $quote = $this->getQuote();
        $shippingAddress = $quote->getShippingAddress();

        if (empty($shippingMethod)) {
            $shippingAddress->unsetShippingMethod();
        }

        $rate = $shippingAddress->getShippingRateByCode($shippingMethod);
        if (!$rate) {
            throw new Creativestyle_AmazonCheckout_Exception_Runtime_Checkout(Mage::helper('checkout')->__('Invalid shipping method.'));
        }

        try {
            $shippingAddress->setShippingMethod($shippingMethod);
            $quote->collectTotals()->save();
        } catch (Exception $e) {
            throw new Creativestyle_AmazonCheckout_Exception_Runtime_Checkout('Error when saving shipping method: '. $e->getMessage());
        }
    }

    /**
     * @param string $couponCode
     * @return Mage_Sales_Model_Quote
     * @api
     */
    public function setCouponCode($couponCode)
    {
        $this->getQuote()->getShippingAddress()->setCollectShippingRates(true);
        return $this->getQuote()->setCouponCode($couponCode)
            ->collectTotals()
            ->save();
    }

    /**
     * @param array $customFields
     * @return Mage_Sales_Model_Quote
     * @api
     */
    public function setCustomFields($customFields)
    {
        $transaction = Mage::getModel('core/resource_transaction');
        $transaction->addObject($this->getQuote());

        foreach ($customFields as $customFieldName => $customFieldValue) {
            if (is_array($customFieldValue)) {
                foreach ($customFieldValue as $key => $value) {
                    switch ($customFieldName) {
                        case 'customer':
                            $transaction->addObject($this->getQuote()->getCustomer());
                            $this->getQuote()->getCustomer()->setData($key, $value);
                            break;
                        case 'billing_address':
                            $transaction->addObject($this->getQuote()->getBillingAddress());
                            $this->getQuote()->getBillingAddress()->setData($key, $value);
                            break;
                        case 'shipping_address':
                            $transaction->addObject($this->getQuote()->getShippingAddress());
                            $this->getQuote()->getShippingAddress()->setData($key, $value);
                            break;
                        default:
                            break;
                    }
                }
            } else {
                $this->getQuote()->setData($customFieldName, $customFieldValue);
            }
        }

        $this->getQuote()->collectTotals();
        $transaction->save();
        return $this->getQuote();
    }

    public function completeOrder()
    {
        $checkoutSession = $this->getAmazonCheckoutSession();

        $this->getQuote()->getPayment()
            ->setTransactionId($checkoutSession->getCheckoutSessionId());

        $this->getQuote()->collectTotals();
        $this->_prepareGuestQuote();

        /** @var Creativestyle_AmazonCheckout_Model_Service_Quote $service */
        $service = Mage::getModel('amazonpay/service_quote', $this->getQuote());
        $service->submitOrder();

        $this->getSession()
            ->setLastQuoteId($this->getQuote()->getId())
            ->setLastSuccessQuoteId($this->getQuote()->getId())
            ->clearHelperData();

        $order = $service->getOrder();

        if ($order) {
            Mage::dispatchEvent(
                'checkout_type_onepage_save_order_after',
                array('order' => $order, 'quote' => $this->getQuote())
            );

            try {
                $order->queueNewOrderEmail();
            } catch (Exception $e) {
                $this->_getLogger()->exception($e);
            }

            // add order information to the session
            $this->getSession()
                ->setLastOrderId($order->getId())
                ->setLastRealOrderId($order->getIncrementId());
        }

        $this->getQuote()->save();

        Mage::dispatchEvent(
            'checkout_submit_all_after',
            array('order' => $order, 'quote' => $this->getQuote(), 'recurring_profiles' => null)
        );

        return $this;
    }

    /**
     * @param string $error
     * @return $this
     */
    public function addSessionError($error)
    {
        $this->getSession()->addError($error);
        return $this;
    }
}
