<?php
/**
 * This file is part of the official Amazon Checkout extension for Magento 1.x
 *
 * (c) 2020 creativestyle GmbH. All Rights reserved
 *
 * Distribution of the derivatives reusing, transforming or being built upon
 * this software, is not allowed without explicit written permission granted
 * by creativestyle GmbH
 *
 * @category   Creativestyle
 * @package    Creativestyle_AmazonCheckout
 * @copyright  2020 creativestyle GmbH
 * @author     Marek Stefan <ticket@creativestyle.de>
 */

class Creativestyle_AmazonCheckout_Model_Observer
{
    /**
     * Return instance of Amazon Pay config object
     *
     * @return Creativestyle_AmazonCheckout_Model_Config
     */
    protected function _getConfig()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Config $config */
        $config = Mage::getSingleton('amazonpay/config');
        return $config;
    }

    /**
     * Return instance of Amazon Pay API adapter
     *
     * @return Creativestyle_AmazonCheckout_Model_Api
     */
    protected function _getApi()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Api $api */
        $api = Mage::getSingleton('amazonpay/api');
        return $api;
    }

    /**
     * Return instance of Amazon Pay helper
     *
     * @return Creativestyle_AmazonCheckout_Helper_Data
     */
    protected function _getAmazonHelper()
    {
        /** @var Creativestyle_AmazonCheckout_Helper_Data $helper */
        $helper = Mage::helper('amazonpay');
        return $helper;
    }

    /**
     * @return Creativestyle_AmazonCheckout_Model_Logger
     */
    protected function _getLogger()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Logger $logger */
        $logger = Mage::getSingleton('amazonpay/logger');
        return $logger;
    }

    /**
     * @param string $method
     * @return bool
     */
    protected function _isAmazonPayMethod($method)
    {
        return in_array($method, $this->_getAmazonHelper()->getAvailablePaymentMethods());
    }

    /**
     * @param string|array $carrierCodes
     * @param mixed|null $store
     * @return string|null
     */
    protected function _mapCarrierCode($carrierCodes, $store = null)
    {
        $carrierCodesMap = $this->_getConfig()->getDeliveryNotificationCarriers($store);

        if (!is_array($carrierCodes)) {
            $carrierCodes = array($carrierCodes);
        }

        foreach ($carrierCodes as $carrierCode) {
            if (isset($carrierCodesMap[strtolower($carrierCode)])) {
                return $carrierCodesMap[strtolower($carrierCode)];
            }
        }

        return null;
    }

    /**
     * @param Varien_Event_Observer $observer
     * @return $this
     */
    public function logApiCall(Varien_Event_Observer $observer)
    {
        $callData = $observer->getEvent()->getCallData();
        $this->_getLogger()->apiCall($callData);
        return $this;
    }

    /**
     * @param Varien_Event_Observer $observer
     * @return $this
     */
    public function sendDeliveryNotification($observer)
    {
        try {
            /** @var Mage_Sales_Model_Order_Shipment $shipment */
            $shipment = $observer->getEvent()->getShipment();

            if ($shipment && $this->_isAmazonPayMethod($shipment->getOrder()->getPayment()->getMethod())
                && $this->_getConfig()->isDeliveryNotificationActive($shipment->getStoreId())) {

                /** @var Mage_Sales_Model_Resource_Order_Shipment_Track_Collection $tracks */
                $tracks = $shipment->getTracksCollection();

                if ($tracks->count()) {
                    /** @var Mage_Sales_Model_Order_Shipment_Track $track */
                    $track  = $tracks->getLastItem();

                    $chargePermissionId = $shipment->getOrder()->getExtOrderId();
                    $trackingNumber = $track->getTrackNumber();
                    $carrierCode = $this->_mapCarrierCode(
                        array(
                            $track->getCarrierCode(),
                            $shipment->getOrder()->getShippingMethod(true)->getCarrierCode()
                        ),
                        $shipment->getStoreId()
                    );

                    if ($carrierCode) {
                        $this->_getApi()->deliveryTrackers($chargePermissionId, $trackingNumber, $carrierCode);
                    }
                }
            }
        } catch (Creativestyle_AmazonCheckout_Exception $e) {
            $this->_getLogger()->exception($e);
        } catch (Exception $e) {
            Mage::logException($e);
        }

        return $this;
    }
}
